@@ -8,7 +8,7 @@
 
 int __fpclassifyf (float _x)
 {
-#ifdef __x86_64__
+#if defined(__x86_64__) || defined(_AMD64_) || defined(__arm__) || defined(_ARM_)
     __mingw_flt_type_t hlp;
 
     hlp.x = _x;
@@ -20,7 +20,7 @@ int __fpclassifyf (float _x)
     if (hlp.val >= 0x7f800000)
       return (hlp.val > 0x7f800000 ? FP_NAN : FP_INFINITE);
     return FP_NORMAL;
-#else
+#elif defined(__i386__) || defined(_X86_)
   unsigned short sw;
   __asm__ __volatile__ (
 	"fxam; fstsw %%ax;"
