@@ -6,8 +6,6 @@
 #include <fenv.h>
 #include <math.h>
 #include <errno.h>
-#define FE_ROUNDING_MASK \
-  (FE_TONEAREST | FE_DOWNWARD | FE_UPWARD | FE_TOWARDZERO)
 
 float
 modff (float value, float* iptr)
@@ -15,7 +13,7 @@ modff (float value, float* iptr)
   float int_part = 0.0F;
   /* truncate */ 
   /* truncate */
-#ifdef _WIN64
+#if defined(_AMD64_) || defined(__x86_64__)
   asm ("subq $8, %%rsp\n"
     "fnstcw 4(%%rsp)\n"
     "movzwl 4(%%rsp), %%eax\n"
@@ -25,7 +23,7 @@ modff (float value, float* iptr)
     "frndint\n"
     "fldcw 4(%%rsp)\n"
     "addq $8, %%rsp\n" : "=t" (int_part) : "0" (value)); /* round */
-#else
+#elif defined(_X86_) || defined(__i386__)
   asm ("push %%eax\n\tsubl $8, %%esp\n"
     "fnstcw 4(%%esp)\n"
     "movzwl 4(%%esp), %%eax\n"
@@ -35,6 +33,8 @@ modff (float value, float* iptr)
     "frndint\n"
     "fldcw 4(%%esp)\n"
     "addl $8, %%esp\n\tpop %%eax\n" : "=t" (int_part) : "0" (value)); /* round */
+#else
+  int_part = truncf(value);
 #endif
   if (iptr)
     *iptr = int_part;
