@@ -5,7 +5,9 @@
  */
 #include <fenv.h>
 
+#if !(defined(_ARM_) || defined(__arm__))
 extern int __mingw_has_sse (void);
+#endif /* !(defined(_ARM_) || defined(__arm__)) */
 
 /* 7.6.2.2  
    The fegetexceptflag function stores an implementation-defined
@@ -14,6 +16,11 @@ extern int __mingw_has_sse (void);
 
 int fegetexceptflag (fexcept_t * flagp, int excepts)
 {
+#if defined(_ARM_) || defined(__arm__)
+  fenv_t _env;
+  __asm__ volatile ("fmrx %0, FPSCR" : "=r" (_env));
+  *flagp = _env.__cw & excepts & FE_ALL_EXCEPT;
+#else
   int _mxcsr;
   unsigned short _status;
 
@@ -23,5 +30,6 @@ int fegetexceptflag (fexcept_t * flagp, int excepts)
     __asm__ volatile ("stmxcsr %0" : "=m" (_mxcsr));
     
   *flagp = (_mxcsr | _status) & excepts & FE_ALL_EXCEPT;
+#endif /* defined(_ARM_) || defined(__arm__) */
   return 0;
 }
