@@ -1,8 +1,13 @@
 /**
- * This file has no copyright assigned and is placed in the Public Domain.
- * This file is part of the w64 mingw-runtime package.
- * No warranty is given; refer to the file DISCLAIMER.PD within this package.
+ * This file is part of the mingw-w64 runtime package.
+ * No warranty is given; refer to the file DISCLAIMER within this package.
  */
+
+#include <winapifamily.h>
+#include <_mingw_unicode.h>
+
+#if WINAPI_FAMILY_PARTITION (WINAPI_PARTITION_DESKTOP)
+
 #ifndef NTLMSP_NAME_A
 #define NTLMSP_NAME_A "NTLM"
 #define NTLMSP_NAME L"NTLM"
@@ -22,19 +27,16 @@
 #define NEGOSSP_NAME_W L"Negotiate"
 #define NEGOSSP_NAME_A "Negotiate"
 
-#ifdef UNICODE
-#define NEGOSSP_NAME NEGOSSP_NAME_W
-#else
-#define NEGOSSP_NAME NEGOSSP_NAME_A
-#endif
+#define NEGOSSP_NAME __MINGW_NAME_UAW(NEGOSSP_NAME)
 #endif
 
 #include <sspi.h>
 
-#if defined(SECURITY_WIN32) || defined(SECURITY_KERNEL)
+#if defined (SECURITY_WIN32) || defined (SECURITY_KERNEL)
 #include <secext.h>
 #endif
 
-#if ISSP_LEVEL==16
+#if ISSP_LEVEL == 16
 #include <issper16.h>
 #endif
+#endif
